@@ -132,15 +132,34 @@ void ImgWriterOgr::close(void)
 void ImgWriterOgr::setCodec(const std::string& imageType){
   //register the drivers
   OGRRegisterAll();
-  //fetch the shape file driver
+  //fetch the OGR file driver
   OGRSFDriver *poDriver;
   poDriver = OGRSFDriverRegistrar::GetRegistrar()->GetDriverByName(imageType.c_str());
   if( poDriver == NULL ){
     std::string errorString="FileOpenError";
     throw(errorString);
   }
-  //create the data source
-  m_datasource=poDriver->CreateDataSource(m_filename.c_str(),NULL);
+  m_datasource = OGRSFDriverRegistrar::Open( m_filename.c_str(), TRUE );
+  if( m_datasource == NULL ){
+    m_datasource = OGRSFDriverRegistrar::Open( m_filename.c_str(), FALSE );
+    if ( m_datasource != NULL){// we can only open in not update mode
+      std::string errorString="Update mode not supported, delete output dataset first";
+      throw(errorString);
+      OGRDataSource::DestroyDataSource(m_datasource);
+      m_datasource = NULL;
+    }
+    else //create the data source
+      m_datasource=poDriver->CreateDataSource(m_filename.c_str(),NULL);
+  }
+  else{//datasets exists, always overwrite all layers (no update append for now)
+    int nLayerCount = m_datasource->GetLayerCount();
+    for(int iLayer = 0; iLayer < nLayerCount; ++iLayer){
+      if(m_datasource->DeleteLayer(iLayer)!=OGRERR_NONE){
+	std::string errorstring="DeleteLayer() failed when overwrite requested";
+	throw(errorstring);
+      }
+    }
+  }
   if(m_datasource==NULL){
     std::string errorString="Creation of output file failed";
     throw(errorString);
@@ -153,8 +172,27 @@ void ImgWriterOgr::setCodec(OGRSFDriver *poDriver){
     std::string errorString="FileOpenError";
     throw(errorString);
   }
-  //create the data source
-  m_datasource=poDriver->CreateDataSource(m_filename.c_str(),NULL);
+  m_datasource = OGRSFDriverRegistrar::Open( m_filename.c_str(), TRUE );
+  if( m_datasource == NULL ){
+    m_datasource = OGRSFDriverRegistrar::Open( m_filename.c_str(), FALSE );
+    if ( m_datasource != NULL){// we can only open in not update mode
+      std::string errorString="Update mode not supported, delete output dataset first";
+      throw(errorString);
+      OGRDataSource::DestroyDataSource(m_datasource);
+      m_datasource = NULL;
+    }
+    else //create the data source
+      m_datasource=poDriver->CreateDataSource(m_filename.c_str(),NULL);
+  }
+  else{//datasets exists, always overwrite all layers (no update append for now)
+    int nLayerCount = m_datasource->GetLayerCount();
+    for(int iLayer = 0; iLayer < nLayerCount; ++iLayer){
+      if(m_datasource->DeleteLayer(iLayer)!=OGRERR_NONE){
+	std::string errorstring="DeleteLayer() failed when overwrite requested";
+	throw(errorstring);
+      }
+    }
+  }
   if(m_datasource==NULL){
     std::string errorString="Creation of output file failed";
     throw(errorString);
@@ -176,6 +214,7 @@ OGRLayer* ImgWriterOgr::createLayer(const std::string& layername, const std::str
   //if points: use wkbPoint
   //if no constraints on the types geometry to be written: use wkbUnknown 
   OGRLayer* poLayer;
+
   OGRSpatialReference oSRS;
 
   if(theProjection!=""){
@@ -291,7 +330,7 @@ void ImgWriterOgr::addPoint(double x, double y, const std::map<std::string,doubl
   pt.setY(y);
   poFeature->SetGeometry( &pt );
   if(createFeature(poFeature,layer)!=OGRERR_NONE){
-    std::string errorString="Failed to create feature in shapefile";
+    std::string errorString="Failed to create feature";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
@@ -316,7 +355,7 @@ void ImgWriterOgr::addPoint(double x, double y, const std::map<std::string,doubl
   pt.setY(y);
   poFeature->SetGeometry( &pt );
   if(createFeature(poFeature,layer)!=OGRERR_NONE){
-    std::string errorString="Failed to create feature in shapefile";
+    std::string errorString="Failed to create feature";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
@@ -337,7 +376,7 @@ void ImgWriterOgr::addLineString(std::vector<OGRPoint*>& points, const std::stri
     throw(errorString);
   }
   if(createFeature(poFeature,layer)!=OGRERR_NONE){
-    std::string errorString="Failed to create feature in shapefile";
+    std::string errorString="Failed to create feature";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
@@ -361,7 +400,7 @@ void ImgWriterOgr::addRing(std::vector<OGRPoint*>& points, const std::string& fi
   // SetSpatialFilter(&thePolygon)
   poFeature->SetGeometry( &thePolygon );
   if(createFeature(poFeature,layer)!=OGRERR_NONE){
-    std::string errorString="Failed to create feature in shapefile";
+    std::string errorString="Failed to create feature";
     throw(errorString);
     OGRFeature::DestroyFeature( poFeature );
   }
@@ -386,7 +425,7 @@ void ImgWriterOgr::addLineString(std::vector<OGRPoint*>& points, const std::stri
     throw(errorString);
   }
   if(createFeature(poFeature,layer)!=OGRERR_NONE){
-    std::string errorString="Failed to create feature in shapefile";
+    std::string errorString="Failed to create feature";
     throw(errorString);
   }
   OGRFeature::DestroyFeature( poFeature );
@@ -493,7 +532,7 @@ int ImgWriterOgr::ascii2ogr(const std::string& filename, const std::string &laye
       if(eGType==wkbPoint){
         pointFeature->SetGeometry( &thePoint );
         if(createFeature(pointFeature)!=OGRERR_NONE){
-          std::string errorString="Failed to create feature in shapefile";
+          std::string errorString="Failed to create feature";
           throw(errorString);
           OGRFeature::DestroyFeature( pointFeature );
         }
@@ -557,7 +596,7 @@ int ImgWriterOgr::ascii2ogr(const std::string& filename, const std::string &laye
       if(eGType==wkbPoint){
         pointFeature->SetGeometry( &thePoint );
         if(createFeature(pointFeature)!=OGRERR_NONE){
-          std::string errorString="Failed to create feature in shapefile";
+          std::string errorString="Failed to create feature";
           throw(errorString);
           OGRFeature::DestroyFeature( pointFeature );
         }
@@ -576,7 +615,7 @@ int ImgWriterOgr::ascii2ogr(const std::string& filename, const std::string &laye
     // SetSpatialFilter(&thePolygon)
     polyFeature->SetGeometry( &thePolygon );
     if(createFeature(polyFeature)!=OGRERR_NONE){
-      std::string errorString="Failed to create feature in shapefile";
+      std::string errorString="Failed to create feature";
       throw(errorString);
       OGRFeature::DestroyFeature( polyFeature );
     }
